#include "stdafx.h"
#include "MfcCDDB.h"
#include <array>


int _tmain()
{
  //Initialize Winsock v2.0
  WSADATA wsaData{};
  const int nWSAStartup{WSAStartup(MAKEWORD(2, 0), &wsaData)};
  if (nWSAStartup != 0)
    return 1;

  try
  {
    //Create the CDDB instance
    CDDB::CClient cddb;

    //Test out the ParseRecordBodyHelper method
    ATL::CAtlFile file;
#pragma warning(suppress: 26477)
    ATLVERIFY(SUCCEEDED(file.Create(_T("cddbrecord.txt"), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, OPEN_EXISTING)));
    ULONGLONG dwLength{0};
#pragma warning(suppress: 26477)
    ATLVERIFY(SUCCEEDED(file.GetSize(dwLength)));
#pragma warning(suppress: 26477)
    ATLVERIFY(dwLength < 1024*1024); //NOLINT(clang-diagnostic-unused-value) Limit our memory allocation to 1 MB for testing purposes
#pragma warning(suppress: 26472)
    std::vector<char> fileData{static_cast<size_t>(dwLength + 1), std::allocator<char>{}};
#pragma warning(suppress: 26472 26486)
    file.Read(fileData.data(), static_cast<UINT>(dwLength));
#pragma warning(suppress: 26446)
    fileData[static_cast<size_t>(dwLength)] = '\0'; //NOLINT(clang-analyzer-core.NullDereference)
    CDDB::CRecord recordtoRead;
#pragma warning(suppress: 26477 26486)
    ATLVERIFY(cddb.ParseRecordBodyHelper(fileData.data(), recordtoRead));
    file.Close();

    //Display all the album details
    _tprintf(_T("\nRetrieved the following album details\n"));
#pragma warning(suppress: 26489)
    _tprintf(_T("Album Title: %s\n"), recordtoRead.m_sTitle.c_str());
#pragma warning(suppress: 26489)
    _tprintf(_T("Album Artist: %s\n"), recordtoRead.m_sArtist.c_str());
#pragma warning(suppress: 26489)
    _tprintf(_T("Submitted via: %s %s %s\n"), recordtoRead.m_sClientName.c_str(), recordtoRead.m_sClientVersion.c_str(), recordtoRead.m_sClientComments.c_str());
    _tprintf(_T("Disc Length (seconds): %d\n"), recordtoRead.m_nDiskLength);
    _tprintf(_T("Database Revision: %d\n"), recordtoRead.m_nDatabaseRevision);
    _tprintf(_T("DISCID: %08x\n"), recordtoRead.m_dwDiscID); //NOLINT(clang-diagnostic-format)

    //Display the track names
    _tprintf(_T("\nTrack Names are:\n"));
#pragma warning(suppress: 26489)
    for (const auto& title : recordtoRead.m_TrackTitles)
#pragma warning(suppress: 26489)
      _tprintf(_T("Track: %s\n"), title.c_str());

    //Display the Extended track data 
    _tprintf(_T("\nExtended Track Data:\n"));
#pragma warning(suppress: 26489)
    for (const auto& trackData : recordtoRead.m_ExtendedTrackData)
#pragma warning(suppress: 26489)
      _tprintf(_T("Track data: %s\n"), trackData.c_str());

    //Display the Extended data 
#pragma warning(suppress: 26489)
    _tprintf(_T("\nExtended Data: %s\n"), recordtoRead.m_ExtendedData.c_str());

    //Display the track offsets 
    _tprintf(_T("\nTrack Offsets:\n"));
#pragma warning(suppress: 26489)
    for (const auto offset : recordtoRead.m_TrackOffsets)
      _tprintf(_T("%d\n"), offset);

    //Display the Playorder
    _tprintf(_T("Playorder :\n"));
#pragma warning(suppress: 26489)
    for (const auto order : recordtoRead.m_PlayOrder)
      _tprintf(_T("%d\n"), order);

    //Try to do a submit with recordtoRead
    CDDB::CSite submitsite{false};
#pragma warning(suppress: 26489)
    submitsite.m_sSite = _T("gnudb.gnudb.org");
#pragma warning(suppress: 26486 26489)
    cddb.Submit(submitsite, recordtoRead.m_sGenre, _T("pjna@naughter.com"), recordtoRead, _T("Test submission by MfcCDDB code"), false);


    //Test out getting all the CDROM drive letters
    std::vector<CDDB::String> drives;
    cddb.GetCDROMDrives(drives);

    _tprintf(_T("CDROM Drives on this machine are:\n"));
    for (const auto& drive : drives)
      _tprintf(_T("%s\n"), drive.c_str());
    if (drives.size() == 0)
    {
      _tprintf(_T("There are no CD drives on this computer\n"));
      WSACleanup();
      return 1;
    }

    //Test out the DISCID function
    DWORD dwDiscID{0};
    _tprintf(_T("Please insert an Audio CD into the first CD drive>\n"));
    const int nChar{_getch()};
    UNREFERENCED_PARAMETER(nChar);
    try
    {
#pragma warning(suppress: 26446)
      dwDiscID = cddb.ComputeDiscIDUsingMCI(drives[0].c_str());
      _tprintf(_T("The CDDB DISCID (using MCI) for this Audio CD is %08x\n"), dwDiscID); //NOLINT(clang-diagnostic-format)
#pragma warning(suppress: 26446)
      dwDiscID = cddb.ComputeDiscIDUsingDeviceIoControl(drives[0].c_str());
      _tprintf(_T("The CDDB DISCID (using DeviceIoControl) for this Audio CD is %08x\n"), dwDiscID); //NOLINT(clang-diagnostic-format)
    }
    catch(CDDB::CException& e)
    {
      std::array<TCHAR, 1024> sMsg{};
#pragma warning(suppress: 26472)
      e.GetErrorMessage(sMsg.data(), static_cast<int>(sMsg.size()));
#pragma warning(suppress: 26472)
      _tprintf(_T("An error occurred while retrieving the DISC ID, Error:%08X, Text:%s\n"), static_cast<unsigned int>(e.m_hr), sMsg.data());
    }

    //Test out the Sites command
    std::vector<CDDB::CSite> sites;
    cddb.Sites(sites);

    _tprintf(_T("Retrieved the following CDDB sites from the CDDB server\n"));
    CDDB::CSite site2;
    bool bFoundFirstPort80{false};
    for (const auto& s : sites)
    {
      //Hive away the first site which works on port 80
      if (!bFoundFirstPort80 && (s.m_nPort == 80))
      {
        bFoundFirstPort80 = true;
#pragma warning(suppress: 26486)
        site2 = s;
      }
    }

    if (!sites.size())
    {
      WSACleanup();
      return 1;
    }
    if (!bFoundFirstPort80)
    {
      WSACleanup();
      return 1;
    }

    //Try the other version of the site function
#pragma warning(suppress: 26486)
    cddb.Sites(site2, sites);

    _tprintf(_T("Retrieved the following CDDB sites from %s\n"), site2.m_sSite.c_str());
    for (const auto& s : sites)
#pragma warning(suppress: 26489)
      _tprintf(_T("%s, %d, %s, %s\n"), s.m_sSite.c_str(), static_cast<int>(s.m_nPort), s.m_sAddress.c_str(), s.m_sDescription.c_str());

    //Try out the message of the day function
    CDDB::String sMessage;
    cddb.MessageOfTheDay(site2, sMessage);

    //Try the Categories function
    std::vector<CDDB::String> categories;
    CDDB::CSite site5;
#pragma warning(suppress: 26489)
    site5.m_sSite = _T("gnudb.gnudb.org");
#pragma warning(suppress: 26486)
    cddb.Categories(site5, categories);
    _tprintf(_T("Retrieved the following CDDB categories from %s\n"), site5.m_sSite.c_str());
    for (const auto& category : categories)
      _tprintf(_T("%s\n"), category.c_str());

    //Try out the Status function
    CDDB::CSite site3{true};
#pragma warning(suppress: 26489)
    site3.m_sSite = _T("gnudb.gnudb.org");
    CDDB::CStatus status;
#pragma warning(suppress: 26486)
    cddb.Status(site3, status);
    //Display the results
    _tprintf(_T("\nStatus of the site %s is as follows\n"), site3.m_sSite.c_str());
    _tprintf(_T("Current protocol: %d\n"), status.m_nCurrentProtocol);
    _tprintf(_T("Max protocol: %d\n"), status.m_nMaxProtocol);
    _tprintf(_T("Gets allowed: %d\n"), static_cast<int>(status.m_bGetsAllowed));
    _tprintf(_T("Updates allowed: %d\n"), static_cast<int>(status.m_bUpdatesAllowed));
    _tprintf(_T("Posting: %d\n"), static_cast<int>(status.m_bPostingAllowed));
    _tprintf(_T("Quotes: %d\n"), static_cast<int>(status.m_bQuotes));
    _tprintf(_T("Current Users: %d\n"), status.m_nCurrentUsers);
    _tprintf(_T("Max Users: %d\n"), status.m_nMaxUsers);
    _tprintf(_T("Strip Extended: %d\n"), static_cast<int>(status.m_bStripExtended));
    _tprintf(_T("Database Entries: %d\n"), status.m_nDatabaseEntries);
    for (size_t i=0; i<status.m_Categories.size(); i++)
#pragma warning(suppress: 26446)
      _tprintf(_T("%s has %d entries\n"), status.m_Categories[i].c_str(), status.m_CategoryEntries[i]);
    for (size_t i=0; i<status.m_PendingSites.size(); i++)
#pragma warning(suppress: 26446)
      _tprintf(_T("%s has %d pending entries\n"), status.m_PendingSites[i].c_str(), status.m_PendingEntries[i]);

    //Test out the GetTrackPositionsUsingMCI function
    std::vector<CDDB::CTrackPosition> tracks;
#pragma warning(suppress: 26446)
    cddb.GetTrackPositionsUsingMCI(drives[0].c_str(), tracks);

    //Test out the GetTrackPositionsUsingDeviceIOControl function
#pragma warning(suppress: 26446)
    cddb.GetTrackPositionsUsingDeviceIoControl(drives[0].c_str(), tracks);

    //Test out the ComputeDiscID function
#pragma warning(suppress: 26446)
    dwDiscID = cddb.ComputeDiscIDUsingMCI(drives[0].c_str());

    //Test out the query function
    std::vector<CDDB::CQueryResult> results;
    cddb.Query(site3, dwDiscID, tracks, results);

    _tprintf(_T("Retrieved the following matches for the inserted audio CD\n"));
    for (const auto& result : results)
      _tprintf(_T("%08x, %s / %s\n"), result.m_dwDiscID, result.m_sArtist.c_str(), result.m_sTitle.c_str()); //NOLINT(clang-diagnostic-format)

    //Try out the read function
    if (results.size())
    {
#pragma warning(suppress: 26446)
      const CDDB::CQueryResult& result{results[0]};
      CDDB::CRecord record;
      cddb.Read(site3, result.m_dwDiscID, result.m_sCategory, record);

      //Display all the album details
      _tprintf(_T("\nRetrieved the following album details\n"));
      _tprintf(_T("Album Title: %s\n"), record.m_sTitle.c_str());
      _tprintf(_T("Album Artist: %s\n"), record.m_sArtist.c_str());
      _tprintf(_T("Submitted via: %s %s %s\n"), record.m_sClientName.c_str(), record.m_sClientVersion.c_str(), record.m_sClientComments.c_str());
      _tprintf(_T("Disc Length (seconds): %d\n"), record.m_nDiskLength);
      _tprintf(_T("Database Revision: %d\n"), record.m_nDatabaseRevision);
      _tprintf(_T("DISCID: %08x\n"), record.m_dwDiscID); //NOLINT(clang-diagnostic-format)

      //Display the track names 
      _tprintf(_T("\nTrack Names are:\n"));
      for (const auto& track : record.m_TrackTitles)
        _tprintf(_T("Track: %s\n"), track.c_str());

      //Display the Extended track data 
      _tprintf(_T("\nExtended Track Data:\n"));
      for (const auto& trackData : record.m_ExtendedTrackData)
        _tprintf(_T("Track data: %s\n"), trackData.c_str());

      //Display the Extended data 
      _tprintf(_T("\nExtended Data: %s\n"), record.m_ExtendedData.c_str());

      //Display the track offsets 
      _tprintf(_T("\nTrack Offsets:\n"));
      for (const auto& trackOffset : record.m_TrackOffsets)
        _tprintf(_T("%d\n"), trackOffset);

      //Display the Playorder
      _tprintf(_T("Playorder :\n"));
      for (const auto& playOrder : record.m_PlayOrder)
        _tprintf(_T("%d\n"), playOrder);

      //Try out the Submit function using the test mode
#pragma warning(suppress: 26477)
      ATLVERIFY(false); //NOLINT(clang-diagnostic-unused-value) please change the email address below to your own to send
                        //any rejection emails to you rather than the made up address.
      CDDB::CSite site6{false};
      site6.m_sSite = _T("gnudb.gnudb.org");
      cddb.Submit(site6, result.m_sCategory, _T("someone@somedomain.com"), record, _T("Test submission by MfcCDDB code"), false);
    }
  }
  catch(CDDB::CException& e)
  {
    std::array<TCHAR, 4096> sMsg{};
#pragma warning(suppress: 26472)
    e.GetErrorMessage(sMsg.data(), static_cast<int>(sMsg.size()));
#pragma warning(suppress: 26472)
    _tprintf(_T("An unexpected CCDDBException has occurred, Error:%08X, Text:%s\n"), static_cast<unsigned int>(e.m_hr), sMsg.data());
  }

  return 0;
}
